@@ -2,8 +2,8 @@
 #
 # simulate.R
 #
-# copyright (c) 2001-2010, Karl W Broman
-# last modified Apr, 2010
+# copyright (c) 2001-2011, Karl W Broman
+# last modified May, 2011
 # first written Apr, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -302,7 +302,7 @@ function(map,model,n.ind,error.prob,missing.prob,
   # add errors
   if(error.prob > 0) {
     for(i in 1:n.chr) {
-      a <- sample(0:1,n.mar[i]*n.ind,repl=TRUE,
+      a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                   prob=c(1-error.prob,error.prob))
       geno[[i]]$data[a == 1] <- 3 - geno[[i]]$data[a == 1]
       if(keep.errorind) {
@@ -320,14 +320,14 @@ function(map,model,n.ind,error.prob,missing.prob,
       o <- grep("^QTL[0-9]+",mar.names[[i]])
       if(length(o)>0)
         x <- geno[[i]]$data[,o]
-      geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,repl=TRUE,
+      geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                             prob=c(missing.prob,1-missing.prob))] <- NA
       if(length(o)>0)
         geno[[i]]$data[,o] <- x
     }
   }
 
-  pheno <- data.frame(phenotype=pheno)
+  pheno <- data.frame(phenotype=pheno, stringsAsFactors=TRUE)
 
   cross <- list(geno=geno,pheno=pheno)
   class(cross) <- c("bc","cross")
@@ -443,12 +443,12 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
   if(error.prob > 0) {
     for(i in 1:n.chr) {
       if(chr.type[i]=="X") {
-        a <- sample(0:1,n.mar[i]*n.ind,repl=TRUE,
+        a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                     prob=c(1-error.prob,error.prob))
         geno[[i]]$data[a == 1] <- 3 - geno[[i]]$data[a == 1]
       }
       else {
-        a <- sample(0:2,n.mar[i]*n.ind,repl=TRUE,
+        a <- sample(0:2,n.mar[i]*n.ind,replace=TRUE,
                     prob=c(1-error.prob,error.prob/2,error.prob/2))
         if(any(a>0 & geno[[i]]$data==1))
           geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
@@ -477,7 +477,7 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
   if(partial.missing.prob > 0) {
     for(i in 1:n.chr) {
       if(chr.type[i] != "X") {
-        o <- sample(c(TRUE,FALSE),n.mar[i],repl=TRUE,
+        o <- sample(c(TRUE,FALSE),n.mar[i],replace=TRUE,
                     prob=c(partial.missing.prob,1-partial.missing.prob))
         if(any(o)) {
           o2 <- grep("^QTL[0-9]+",mar.names[[i]])
@@ -504,14 +504,14 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
       o <- grep("^QTL[0-9]+",mar.names[[i]])
       if(length(o)>0)
         x <- geno[[i]]$data[,o]
-      geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,repl=TRUE,
+      geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                             prob=c(missing.prob,1-missing.prob))] <- NA
       if(length(o)>0)
         geno[[i]]$data[,o] <- x
     }
   }
 
-  pheno <- data.frame(phenotype=pheno)
+  pheno <- data.frame(phenotype=pheno, stringsAsFactors=TRUE)
 
   cross <- list(geno=geno,pheno=pheno)
   class(cross) <- c("f2","cross")
@@ -645,7 +645,7 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
   if(error.prob > 0) {
     for(i in 1:n.chr) {
       if(chr.type[i] != "X") { # 4-way cross; autosomal
-        a <- sample(0:3,n.mar[i]*n.ind,repl=TRUE,
+        a <- sample(0:3,n.mar[i]*n.ind,replace=TRUE,
                     prob=c(1-error.prob,rep(error.prob/3,3)))
         if(any(a>0 & geno[[i]]$data==1))
           geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
@@ -661,7 +661,7 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
             geno[[i]]$data[a>0 & geno[[i]]$data==4] - a[a>0 & geno[[i]]$data==4]
       }
       else {
-        a <- sample(0:1,n.mar[i]*n.ind,repl=TRUE,
+        a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                     prob=c(1-error.prob,error.prob))
         if(any(a>0 & geno[[i]]$data==1))
           geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
@@ -691,7 +691,7 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
   if(partial.missing.prob > 0) {
     for(i in 1:n.chr) {
       if(chr.type[i] != "X") {
-        o <- sample(c(TRUE,FALSE),n.mar[i],repl=TRUE,
+        o <- sample(c(TRUE,FALSE),n.mar[i],replace=TRUE,
                     prob=c(partial.missing.prob,1-partial.missing.prob))
 
         if(any(o)) {
@@ -728,7 +728,7 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
       o <- grep("^QTL[0-9]+",mar.names[[i]])
       if(length(o)>0)
         x <- geno[[i]]$data[,o]
-      geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,repl=TRUE,
+      geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                             prob=c(missing.prob,1-missing.prob))] <- NA
       if(length(o)>0)
         geno[[i]]$data[,o] <- x
@@ -744,7 +744,7 @@ function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
     dimnames(pheno) <- list(NULL, "phenotype")
   }
 
-  pheno <- as.data.frame(pheno)
+  pheno <- as.data.frame(pheno, stringsAsFactors=TRUE)
   cross <- list(geno=geno,pheno=pheno)
   class(cross) <- c("4way","cross")
 
