@@ -2,8 +2,8 @@
 #
 # tryallpositions.R
 #
-# copyright (c) 2007-2010, Karl W Broman
-# last modified Jul, 2010
+# copyright (c) 2007-2011, Karl W Broman
+# last modified May, 2011
 # first written Oct, 2007
 #
 #     This program is free software; you can redistribute it and/or
@@ -363,7 +363,7 @@ function(cross, chr, error.prob=0.0001,
     if(n.mar[i] == 2) { # 2 markers
       mmll <- markerloglik(thischr, markernames(thischr), error.prob=error.prob)
       temp <- data.frame(chr=chrnam[i], pos=pos,
-                         lod=(initialloglik - sum(mmll))/log(10), gap=gap)
+                         lod=(initialloglik - sum(mmll))/log(10), gap=gap, stringsAsFactors=TRUE)
       rownames(temp) <- interval
     }
     else { # >2 markers
@@ -408,7 +408,7 @@ function(cross, chr, error.prob=0.0001,
         attr(est.map(drop.markers(thischr, mn[length(mn)]), error.prob=error.prob,
                      map.function=map.function, m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)[[1]], "loglik")
 
-      temp <- data.frame(chr=rep(chrnam[i], length(interval)), pos=pos, lod=lod/log(10), gap=gap)
+      temp <- data.frame(chr=rep(chrnam[i], length(interval)), pos=pos, lod=lod/log(10), gap=gap, stringsAsFactors=TRUE)
       rownames(temp) <- interval
     }
     result <- rbind(result, temp)
